<?php
/*
....
*/
require_once dirname( __FILE__ ).'/base.addon.php';

if( !class_exists( 'CPCFF_HubSpot' ) )
{
    class CPCFF_HubSpot extends CPCFF_BaseAddon
    {
		static public $category = 'External Services';

        /************* ADDON SYSTEM - ATTRIBUTES AND METHODS *************/
		protected $addonID = "addon-hubspot-20191123";
		protected $name = "CFF - HubSpot";
		protected $description;

		public function get_addon_form_settings( $form_id )
		{
			global $wpdb;

			// Insertion in database
			if(
				isset( $_REQUEST[ 'cpcff_hubspot' ] )
			)
			{
				$api_key  = isset($_REQUEST['cpcff_hubspot_api_key']) ? sanitize_text_field($_REQUEST['cpcff_hubspot_api_key']) : '';

				$fields = array(
					"email" => sanitize_text_field($_REQUEST['cpcff_hubspot_email']),
					"firstname" => sanitize_text_field($_REQUEST['cpcff_hubspot_firstname']),
					"lastname" => sanitize_text_field($_REQUEST['cpcff_hubspot_lastname']),
					"website" => sanitize_text_field($_REQUEST['cpcff_hubspot_website']),
					"company" => sanitize_text_field($_REQUEST['cpcff_hubspot_company']),
					"phone"  => sanitize_text_field($_REQUEST['cpcff_hubspot_phone']),
					"address" => sanitize_text_field($_REQUEST['cpcff_hubspot_address']),
					"city" => sanitize_text_field($_REQUEST['cpcff_hubspot_city']),
					"state" => sanitize_text_field($_REQUEST['cpcff_hubspot_state']),
					"zip" => sanitize_text_field($_REQUEST['cpcff_hubspot_zip'])
				);

				$data  = array(
					'api_key' => $api_key,
					'fields' => $fields
				);

				$wpdb->delete( $wpdb->prefix.$this->form_hubspot_table, array( 'formid' => $form_id ), array( '%d' ) );
				$wpdb->insert(
					$wpdb->prefix.$this->form_hubspot_table,
					array(
						'formid'  => $form_id,
						'enabled' => isset($_REQUEST['cpcff_hubspot_enabled']) ? 1 : 0,
						'data'	  => json_encode( $data )
					),
					array( '%d', '%d', '%s' )
				);
			}

			$enabled = false;
			$api_key = '';
			$fields  = array();
			$row  = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_hubspot_table." WHERE formid=%d", $form_id ) );

			if( $row )
			{
				$enabled = $row->enabled;
				$data = json_decode($row->data, true);
				if(!is_null($data))
				{
					$api_key = $data['api_key'];
					$fields = $data['fields'];
				}
			}
			?>
			<input type="hidden" name="cpcff_hubspot" value="1" />
			<div id="metabox_basic_settings" class="postbox" >
				<h3 class='hndle' style="padding:5px;"><span><?php print $this->name; ?></span></h3>
				<div class="inside">
					<table cellspacing="0" style="width:100%;">
						<tr>
							<td style="white-space:nowrap;width:200px;"><?php _e('Enabling HubSpot Integration', 'calculated-fields-form');?>:</td>
							<td>
								<input type="checkbox" name="cpcff_hubspot_enabled" <?php echo ( ( $enabled ) ? 'CHECKED' : '' ); ?> />
							</td>
						</tr>
						<tr>
						<tr>
							<td style="white-space:nowrap;width:200px;"><?php _e('HubSpot api key', 'calculated-fields-form');?>:</td>
							<td>
								<input type="text" name="cpcff_hubspot_api_key" value="<?php echo esc_attr( $api_key ); ?>" style="width:80%;" /> <a href="https://knowledge.hubspot.com/integrations/how-do-i-get-my-hubspot-api-key" target="_blank" style="font-weight:bold;font-size:1.5em;">?</a>
							</td>
						</tr>
						<tr>
							<td cospan="2"><b><?php _e('Fields relationship', 'calculated-fields-form'); ?></b></td>
						</tr>
						<tr>
							<td><?php _e('Email (required)', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_email" value="<?php print esc_attr((isset($fields['email'])) ? $fields['email'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('First name', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_firstname" value="<?php print esc_attr((isset($fields['firstname'])) ? $fields['firstname'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Last name', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_lastname" value="<?php print esc_attr((isset($fields['lastname'])) ? $fields['lastname'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Website', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_website" value="<?php print esc_attr((isset($fields['website'])) ? $fields['website'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Company', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_company" value="<?php print esc_attr((isset($fields['company'])) ? $fields['company'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Phone', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_phone" value="<?php print esc_attr((isset($fields['phone'])) ? $fields['phone'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Address', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_address" value="<?php print esc_attr((isset($fields['address'])) ? $fields['address'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('City', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_city" value="<?php print esc_attr((isset($fields['city'])) ? $fields['city'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('State', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_state" value="<?php print esc_attr((isset($fields['state'])) ? $fields['state'] : ''); ?>"></td>
						</tr>
						<tr>
							<td><?php _e('Zip', 'calculated-fields-form'); ?></td>
							<td><input type="text" placeholder="fieldname#" name="cpcff_hubspot_zip" value="<?php print esc_attr((isset($fields['zip'])) ? $fields['zip'] : ''); ?>"></td>
						</tr>
					</table>
				</div>
			</div>
			<?php
		}

		/************************ ADDON CODE *****************************/
        /************************ ATTRIBUTES *****************************/

		private $form_hubspot_table = 'cp_calculated_fields_form_hubspot';
		private $endpoint = 'https://api.hubapi.com/contacts/v1/contact/createOrUpdate/email/%s/?hapikey=%s';

        /************************ CONSTRUCT *****************************/

        function __construct()
        {
			$this->description = __("The add-on allows to create/update HubSpot contacts from the website forms", 'calculated-fields-form' );
            // Check if the plugin is active
			if( !$this->addon_is_active() ) return;

			// Export the lead
			add_action( 'cpcff_process_data', array( &$this, 'send_to_hubspot' ) );

			if( is_admin() )
			{
				// Delete forms
				add_action( 'cpcff_delete_form', array(&$this, 'delete_form') );

				// Clone forms
				add_action( 'cpcff_clone_form', array(&$this, 'clone_form'), 10, 2 );

				// Export addon data
				add_action( 'cpcff_export_addons', array(&$this, 'export_form'), 10, 2 );

				// Import addon data
				add_action( 'cpcff_import_addons', array(&$this, 'import_form'), 10, 2 );
			}
        } // End __construct

        /************************ PROTECTED METHODS *****************************/

		/**
         * Create the database tables
         */
        protected function update_database()
		{
			global $wpdb;
			$charset_collate = $wpdb->get_charset_collate();
			$db_queries = array( "CREATE TABLE ".$wpdb->prefix.$this->form_hubspot_table." (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					formid INT NOT NULL,
					enabled TINYINT DEFAULT 0 NOT NULL,
					data text,
					UNIQUE KEY id (id)
				) $charset_collate;"
			);
			$this->_run_update_database($db_queries);
		} // End update_database

        /************************ PRIVATE METHODS *****************************/

		/************************ PUBLIC METHODS  *****************************/

		/**
         * Send the information to HubSpot
         */
        public function	send_to_hubspot( $params )
		{
			global $wpdb, $wp_version;

			$form_id = @intval( $params[ 'formid' ] );
			if( $form_id )
			{
				$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_hubspot_table." WHERE formid=%d", $form_id ) );
				if( $row && !empty( $row->enabled ) )
				{
					$data = json_decode( $row->data, true);
					if(!empty($data))
					{
						$api_key = $data['api_key'];
						if(!empty($api_key))
						{
							$hubspot_args = array('properties' => array());
							$email = '';

							$fields = (!empty($data['fields']) && is_array($data['fields'])) ? $data['fields'] : array();
							foreach( $fields as $attr => $field )
							{
								$field = trim($field);
								$value = isset($params[ $field ]) ? $params[ $field ] : (preg_match('/fieldname\d+/',$field) ? '' : $field);

								if($attr == 'email')
									$email = $value;
								elseif(!empty($value))
									$hubspot_args['properties'][] = array(
										'property' => $attr,
										'value'	=> $value
									);
							}

							if(!empty($email))
							{
								$url = sprintf($this->endpoint, $email, $api_key);
								$args = array(
									'body' 		=> json_encode( $hubspot_args ),
									'headers' 	=> array(
										'Content-Type' => 'application/json',
										'user-agent' => 'WordPress-to-HubSpot for calculated-fields-form plugin - WordPress/'.$wp_version.'; '.get_bloginfo( 'url' ),
									),
									'timeout' => 45,
									'sslverify'	=> false,
								);
								$response = wp_remote_post($url, $args);
								if ( is_wp_error( $response ) )
								{
									$error_message = $response->get_error_message();
									error_log( "CFF_HubSpot Error: $error_message" );
								}
							}
							else
							{
								error_log( "CFF_HubSpot Error: The email is empty" );
							}
						}
					}
				}
			}
		} // End send_to_hubspot

		/**
		 *	Delete the form from the addon's table
		 */
        public function delete_form( $formid)
		{
			global $wpdb;
			$wpdb->delete( $wpdb->prefix.$this->form_hubspot_table, array('formid' => $formid), '%d' );
		} // delete_form

		/**
		 *	Clone the form's row
		 */
		public function clone_form( $original_form_id, $new_form_id )
		{
			global $wpdb;

			$form_row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_hubspot_table." WHERE formid=%d", $original_form_id ), ARRAY_A);

			if(!empty($form_row))
			{
				unset($form_row["id"]);
				$form_row["formid"] = $new_form_id;
				$wpdb->insert( $wpdb->prefix.$this->form_hubspot_table, $form_row);
			}
		} // End clone_form

		/**
		 *	It is called when the form is exported to export the addons data too.
		 *  Receive an array with the other addons data, and the form's id for filtering.
		 */
		public function export_form($addons_array, $formid)
		{
			global $wpdb;
			$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_hubspot_table." WHERE formid=%d", $formid ), ARRAY_A );
			if(!empty( $row ))
			{
				unset($row['id']);
				unset($row['formid']);
				$addons_array[ $this->addonID ] = $row;
			}
			return $addons_array;
		} // End export_form

		/**
		 *	It is called when the form is imported to import the addons data too.
		 *  Receive an array with all the addons data, and the new form's id.
		 */
		public function import_form($addons_array, $formid)
		{
			global $wpdb;
			if(isset($addons_array[$this->addonID]))
			{
				$addons_array[$this->addonID]['formid'] = $formid;
				$wpdb->insert(
					$wpdb->prefix.$this->form_hubspot_table,
					$addons_array[$this->addonID]
				);
			}
		} // End import_form

    } // End Class

    // Main add-on code
    $cpcff_hubspot_obj = new CPCFF_HubSpot();

	// Add addon object to the objects list
	CPCFF_ADDONS::add($cpcff_hubspot_obj);
}