<?php
/*
....
*/
require_once dirname( __FILE__ ).'/base.addon.php';

if( !class_exists( 'CPCFF_PDF' ) )
{
    class CPCFF_PDF extends CPCFF_BaseAddon
    {
		static public $category = 'Extending Features';

        /************* ADDON SYSTEM - ATTRIBUTES AND METHODS *************/
		protected $addonID = "addon-pdf-20200213";
		protected $name = "CFF - PDF Generator (Experimental)";
		protected $description;

		public function get_addon_form_settings( $form_id )
		{
			global $wpdb;

			// Enqueue the required scripts
			wp_enqueue_code_editor( array( 'type' => 'text/html' ) );
			wp_enqueue_script( 'cpcff_p_js', plugins_url('/pdf.addon/code-editor.js', __FILE__), array( 'jquery' ), $this->addonID, true );

			$table = $wpdb->prefix.$this->form_pdf_table;
			// Insertion in database
			if( isset( $_REQUEST[ 'cpcff_pdf' ] ) )
			{
				$wpdb->delete( $table, array( 'formid' => $form_id ), array( '%d' ) );

				$active = (isset($_REQUEST['cpcff_pdf_active'])) ? 1 : 0;
				$data 	= (isset($_REQUEST['cpcff_pdf_data'])) ? trim($_REQUEST['cpcff_pdf_data']) : '';
				$title 	= (isset($_REQUEST['cpcff_pdf_title'])) ? trim($_REQUEST['cpcff_pdf_title']) : '';
				$valid_options = array('notification-email', 'confirmation-email', 'both-emails');
				$email 	= (
							isset($_REQUEST['cpcff_pdf_email']) &&
							in_array($_REQUEST['cpcff_pdf_email'], $valid_options)
						)
						? $_REQUEST['cpcff_pdf_email']
						: 'both-emails';

				$this->_insert_row(
					$table,
					array(
						'formid' 		=> $form_id,
						'active'		=> $active,
						'title'			=> stripslashes($title),
						'data'			=> stripslashes($data),
						'email'			=> $email
					),
					array( '%d', '%d', '%s', '%s', '%s' )
				);
			}

			$row = $wpdb->get_row(
				$wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_pdf_table." WHERE formid=%d", $form_id )
			);

			$active = false;
			$data 	= '';
			$title 	= '';
			$email  = 'both-emails';

			if(!is_null($row))
			{
				$active = $row->active;
				$data 	= $row->data;
				$title 	= $row->title;
				if(!empty($row->email)) $email = $row->email;
			}
			?>
			<div id="metabox_basic_settings" class="postbox" >
				<h3 class='hndle' style="padding:5px;"><span><?php print $this->name; ?></span></h3>
				<div class="inside">
					<table cellspacing="3px" style="width:100%;">
						<tr>
							<td>
								<input type="checkbox" name="cpcff_pdf_active" <?php if($active) print 'CHECKED'; ?> />
								<?php _e('Activate the PDF add-on', 'calculated-fields-form'); ?>
							</td>
						</tr>
						<tr>
							<td>
								<p><?php _e('Enter the name of PDF file', 'calculated-fields-form'); ?>:</p>
								<input name="cpcff_pdf_title" style="width:100%;" value="<?php print esc_attr($title); ?>" />
							</td>
						</tr>
						<tr>
							<td>
								<p><?php _e('Send the PDF file to', 'calculated-fields-form'); ?>:</p>
								<input name="cpcff_pdf_email" type="radio" value="notification-email" <?php if($email == 'notification-email') print 'CHECKED'; ?> /> <?php _e('To the email addresses entered through the "Destination emails" attribute', 'calculated-fields-form'); ?><br>
								<input name="cpcff_pdf_email" type="radio" value="confirmation-email" <?php if($email == 'confirmation-email') print 'CHECKED'; ?> /> <?php _e('To the email addresses entered by the user', 'calculated-fields-form'); ?><br>
								<input name="cpcff_pdf_email" type="radio" value="both-emails" <?php if($email == 'both-emails') print 'CHECKED'; ?> /> <?php _e('Both emails', 'calculated-fields-form'); ?>
							</td>
						</tr>
						<tr>
							<td>
								<p><?php _e('Enter the HMTL code to generate the PDF file', 'calculated-fields-form'); ?>:</p>
								<textarea name="cpcff_pdf_data" style="width:100%;" rows="20"><?php print esc_textarea($data); ?></textarea>
								<p><?php _e('It is possible to use the same special tags that in notification emails and thank you pages', 'calculated-fields-form'); ?>: <a href="https://cff.dwbooster.com/documentation#special-tags" target="_blank"><?php _e('READ MORE', 'calculated-fields-form'); ?></a></p>
							</td>
						</tr>
					</table>
				</div>
				<input type="hidden" name="cpcff_pdf" value="1" />
			</div>
			<?php
		}

		/************************ ADDON CODE *****************************/
        /************************ ATTRIBUTES *****************************/

		private $form_pdf_table = 'cp_calculated_fields_form_pdf';
		private $pdf_file_content;
		private $pdf_file_name 	= '';

        /************************ CONSTRUCT *****************************/

        function __construct()
        {
			$this->description = __("The add-on generates a PDF file with the information collected by the form, and attach it to the notification and confirmation emails", 'calculated-fields-form');

            // Check if the plugin is active
			if( !$this->addon_is_active() ) return;

			// Export the lead
			add_filter( 'cpcff_notification_email_attachments', array( &$this, 'notification_email' ), 1, 4 );
			add_filter( 'cpcff_confirmation_email_attachments', array( &$this, 'confirmation_email' ), 1, 4 );

			if( is_admin() )
			{
				// Delete forms
				add_action( 'cpcff_delete_form', array(&$this, 'delete_form') );

				// Clone forms
				add_action( 'cpcff_clone_form', array(&$this, 'clone_form'), 10, 2 );

				// Export addon data
				add_action( 'cpcff_export_addons', array(&$this, 'export_form'), 10, 2 );

				// Import addon data
				add_action( 'cpcff_import_addons', array(&$this, 'import_form'), 10, 2 );
			}

        } // End __construct

        /************************ PROTECTED METHODS *****************************/

		/**
         * Creates the database tables
         */
        protected function update_database()
		{
			global $wpdb;
			$charset_collate = $wpdb->get_charset_collate();
			$db_queries = array( "CREATE TABLE ".$wpdb->prefix.$this->form_pdf_table." (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					formid INT NOT NULL,
					active TINYINT DEFAULT 0 NOT NULL,
					title text DEFAULT '' NOT NULL,
					data LONGTEXT DEFAULT '' NOT NULL,
					email VARCHAR(50) DEFAULT 'both-emails' NOT NULL,
					UNIQUE KEY id (id)
				) $charset_collate;"
			);
			$this->_run_update_database($db_queries);
		} // End update_database

        /************************ PRIVATE METHODS *****************************/

		private function _insert_row($table, $columns, $format)
		{
			global $wpdb;
			if(!$wpdb->insert($table, $columns, $formats))
			{
				$this->update_database();
				$wpdb->insert($table, $columns, $formats);
			}
		} // End _insert_row

		private function _get_email($form_id)
		{
			global $wpdb;
			$email = 'both-emails';
			$row = $wpdb->get_row(
				$wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_pdf_table." WHERE formid=%d", $form_id )
			);

		    if(is_null($row) || !$row->active) return false;
			if(!empty($row->email)) $email = $row->email;

			return $email;
		} // End _get_email

		/************************ PUBLIC METHODS  *****************************/

		/**
		 * Attach the generated PDF file to the notification email
		 */
        public function	notification_email( $files, $params, $form_id, $submission_id )
		{
			$email = $this->_get_email($form_id);
			if($email !== false)
			{
				if($email != 'confirmation-email') $this->generate_file( $files, $params, $form_id, $submission_id );
				else remove_action( 'phpmailer_init', array(&$this, 'phpmailer_init') );
			}
			return $files;
		} // End notification_email

        /**
		 * Attach the generated PDF file to the email copy to the user
		 */
        public function	confirmation_email( $files, $params, $form_id, $submission_id )
		{
			$email = $this->_get_email($form_id);
			if($email !== false)
			{
				if($email != 'notification-email') $this->generate_file( $files, $params, $form_id, $submission_id );
				else remove_action( 'phpmailer_init', array(&$this, 'phpmailer_init') );
			}
			return $files;
		} // End confirmation_email

        /**
         * Generate the PDF file
         */
        public function	generate_file( $files, $params, $form_id, $submission_id )
		{
			global $wpdb;
			if(!empty($this->pdf_file_content))
			{
				add_action( 'phpmailer_init', array(&$this, 'phpmailer_init') );
			}
			else
			{
				if(!empty( $form_id ))
				{
					$row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_pdf_table." WHERE formid=%d", $form_id ) );

					if(
						!is_null($row) &&
						$row->active &&
						!empty($row->data)
					)
					{
						$submission_obj = CPCFF_SUBMISSIONS::get($submission_id);
						if($submission_obj)
						{
							$form_obj = CPCFF_SUBMISSIONS::get_form($submission_id);
							$data = CPCFF_AUXILIARY::parsing_fields_on_text(
								$form_obj->get_fields(),
								$params,
								$row->data,
								$submission_obj->data,
								'html',
								$submission_id
							);

							if(!empty($row->title))
							{
								$file_title = CPCFF_AUXILIARY::parsing_fields_on_text(
									$form_obj->get_fields(),
									$params,
									$row->title,
									$submission_obj->data,
									'text',
									$submission_id
								);
								$this->pdf_file_name = $file_title['text'];
							}

							// Generating the PDF file
							require_once dirname(__FILE__).'/pdf.addon/vendor/autoload.php';
							$options = new Dompdf\Options();
							$options->set('isRemoteEnabled', TRUE);
							$options->set('defaultFont', 'helvetica');
							$dompdf = new Dompdf\Dompdf($options);
							$dompdf->loadHtml($data['text']);

							$dompdf->setPaper('A4', 'portrait');
							$dompdf->render();
							$this->pdf_file_content = $dompdf->output();
							add_action( 'phpmailer_init', array(&$this, 'phpmailer_init') );
						}
					}
				}
			}
			return $files;
		} // End generate_file

		public function phpmailer_init(&$phpmailer)
		{
			if(!empty($this->pdf_file_content))
			{
				$phpmailer->addStringAttachment($this->pdf_file_content, $this->pdf_file_name.'.pdf', 'base64', 'application/pdf');
			}
		} // End phpmailer_init

		/**
		 *	Delete the form from the addon's table
		 */
        public function delete_form( $formid)
		{
			global $wpdb;
			$wpdb->delete( $wpdb->prefix.$this->form_pdf_table, array('formid' => $formid), '%d' );
		} // delete_form

		/**
		 *	Clone the form's row
		 */
		public function clone_form( $original_form_id, $new_form_id )
		{
			global $wpdb;

			$rows = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_pdf_table." WHERE formid=%d", $original_form_id ), ARRAY_A);

			if(!empty($rows))
			{
				foreach($rows as $row)
				{
					unset($row["id"]);
					$row["formid"] = $new_form_id;
					$wpdb->insert( $wpdb->prefix.$this->form_pdf_table, $row);
				}
			}
		} // End clone_form

		/**
		 *	It is called when the form is exported to export the addons data too.
		 *  Receive an array with the other addons data, and the form's id for filtering.
		 */
		public function export_form($addons_array, $formid)
		{
			global $wpdb;
			$rows = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM ".$wpdb->prefix.$this->form_pdf_table." WHERE formid=%d", $formid ), ARRAY_A );
			if(!empty( $rows ))
			{
				$addons_array[ $this->addonID ] = array();
				foreach($rows as $row)
				{
					unset($row['id']);
					unset($row['formid']);
					$addons_array[ $this->addonID ][] = $row;
				}
			}
			return $addons_array;
		} // End export_form

		/**
		 *	It is called when the form is imported to import the addons data too.
		 *  Receive an array with all the addons data, and the new form's id.
		 */
		public function import_form($addons_array, $formid)
		{
			global $wpdb;
			if(isset($addons_array[$this->addonID]))
			{
				foreach($addons_array[$this->addonID] as $row)
				{
					if(!empty($row))
					{
						$row['formid'] = $formid;
						$wpdb->insert(
							$wpdb->prefix.$this->form_pdf_table,
							$row
						);
					}
				}
			}
		} // End import_form

    } // End Class

    // Main add-on code
    $cpcff_pdf_obj = new CPCFF_PDF();

	// Add addon object to the objects list
	CPCFF_ADDONS::add($cpcff_pdf_obj);
}