/*
* chart.js v0.1
* By: CALCULATED FIELD PROGRAMMERS
* Allows to use the chart.js library (https://www.chartjs.org/) with the CFF
* Copyright 2019 CODEPEOPLE
*/

;(function(root){
	var lib = {}, _$, _loading = false, _queue = [], colors_json={"red":["#ffa8a8","#ff8787","#ff6b6b","#fa5252"],"blue":["#74c0fc","#4dabf7","#339af0","#228be6"],"yellow":["#ffe066","#ffd43b","#fcc419","#fab005"],"green":["#8ce99a","#69db7c","#51cf66","#40c057"],"pink":["#faa2c1","#f783ac","#f06595","#e64980"],"cyan":["#66d9e8","#3bc9db","#22b8cf","#15aabf"],"orange":["#ffc078","#ffa94d","#ff922b","#fd7e14"],"lime":["#c0eb75","#a9e34b","#94d82d","#82c91e"],"grape":["#e599f7","#da77f2","#cc5de8","#be4bdb"],"indigo":["#91a7ff","#748ffc","#5c7cfa","#4c6ef5"],"teal":["#63e6be","#38d9a9","#20c997","#12b886"],"violet":["#b197fc","#9775fa","#845ef7","#7950f2"]}, _initialized = false;

	/*** PRIVATE FUNCTIONS ***/
	function pickColor(i)
	{
		var idx = Object.keys(colors_json),
			plt = colors_json[idx[i % idx.length]];
		return plt[Math.floor(i % plt.length)];
	};

	function init_and_services()
	{
		if(_initialized) return;
		_initialized = true;

		Chart.defaults.scale.ticks.beginAtZero = true;
		Chart.pluginService.register(
			{
				beforeUpdate: function(chart)
				{
					try
					{
						var i = 0, m = 0;
						if('data' in chart && 'datasets' in chart.data)
						{
							var dss = chart.data.datasets,
								dss_length = dss.length;

							for(var ds in dss)
							{
								ds = dss[ds];
								if(!ds.borderColor)
								{
									var bc = [];
									for(var _d in ds.data)
									{
										_d = ds.data[_d];
										bc.push(pickColor((dss_length < 2) ? i++ : i));
									}
									if(2 <= dss_length) i++;
									ds.borderColor = bc;
									if(!ds.backgroundColor)
									{
										ds.backgroundColor = ds.borderColor.map(
											function(hex)
											{
												var color = new Color(hex).alpha(0.8);
												return color.rgbaString();
											}
										);
									}
								}
							}
						}
					}catch(err){if('console' in window) console.error(err.message);}
				}
			}
		);
	};

	function load_and_clear_queue()
	{
		if(!('Chart' in window))
		{
			if (!_loading)
			{
				_$.getScript(
					'https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.7.3/Chart.min.js',
					function ()
					{
						init_and_services();
						clear_queue();
					}
				);
			}
			_loading = true;
			return;
		}
		init_and_services();
		clear_queue();
	};

	function add_queue(args)
	{
		// Add the to queue
		_queue.push(args);
		load_and_clear_queue();
	};

	function clear_queue()
	{
		var item;
		while(_queue.length)
		{
			// Generate or update the chart
			item = _queue.shift();
			if('id' in item && 'args' in item) generator(item.id, item.args);
		}
	};

	/*** CHART FUNCTIONS ***/

	function generator(id, obj)
	{
		var _canvas = _$('[id="'+id.replace(/['"]/g, '\$1')+'"]'), _chart;
		if(_canvas.length)
		{
			_chart = _canvas.data('chart-obj');
			if(_chart)
			{
				_chart.clear();
				_chart.destroy();
			}
			_chart = new Chart(
				_canvas,
				obj
			);
			_canvas.data('chart-obj', _chart);
		}
	};

	/*** PUBLIC FUNCTIONS ***/

	lib.cff_connector_version = '0.1';

	lib.cffchart = lib.CFFCHART = function(canvasId, args)
	{
		if('undefined' == typeof fbuilderjQuery) return;
		_$ = fbuilderjQuery;
		add_queue(
			{
				'id' 	: canvasId,
				'args' 	: args
			}
		);
	};
	root.CF_CHART = lib;

})(this);